#ifndef __PLUCKED_STR_
#define __PLUCKED_STR_

#include "AudioChainElement.h"
#include "DelayFilter.h"
#include "MovingAvgFilter.h"

// The lowest frequency supported in Hz
#define MIN_STRING_FREQ (10)

#include <math.h>
class PluckedString :
  public AudioChainElement
{
public:
  typedef enum StringState {
    STATE_ACTIVE = 0,
    STATE_RELEASED = 1,
    STATE_FINISHED = 2
  }StringState;

  typedef void (*ReleaseCallback)(PluckedString* pFinishedString, unsigned int userData);

  PluckedString(MyFloat64 freq, unsigned int samplingRate,
                ReleaseCallback callback = NULL, unsigned int callbackUserData = 0);
  ~PluckedString(void);

public:
  virtual SAMPLE generateOutput(SAMPLE input);
  void release(void);

  StringState getState(void) { 
    return m_state;
  }

  bool isFinished() {
    return m_state == STATE_FINISHED;
  }

private:
  int frequencyToDelayLength(MyFloat64 freq, int samplingRate);

  static const MyFloat64 baseNumReleaseSamples;
  
  MyFloat64 m_numReleaseSamples;  
  int m_SamplesSinceRelease;
  DelayFilter* m_pSimpleDelay;
  MovingAvgFilter* m_pAvgFilter;
  
  StringState m_state;
  
  // Callback to call at the end of the release (when moving to state Finished).
  ReleaseCallback m_releaseCallback;
  unsigned int m_callbackUserData;
};

#endif
